@@ -1,7 +1,7 @@
 // synthv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -36,6 +36,8 @@
 #include "synthv1_controls.h"
 #include "synthv1_programs.h"
 
+#include "synthv1_sched.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -729,6 +731,38 @@ struct synthv1_voice : public synthv1_list<synthv1_voice>
 };
 
 
+// MIDI input asynchronous status notification
+
+class synthv1_midi_in : public synthv1_sched
+{
+public:
+
+	synthv1_midi_in (synthv1 *pSynth)
+		: synthv1_sched(pSynth, MidiIn),
+			m_enabled(false), m_count(0) {}
+
+	void schedule_event()
+		{ if (m_enabled && ++m_count < 2) schedule(-1); }
+
+	void process(int) {}
+
+	void enabled(bool on)
+		{ m_enabled = on; m_count = 0; }
+
+	uint32_t count()
+	{
+		const uint32_t ret = m_count;
+		m_count = 0;
+		return ret;
+	}
+
+private:
+
+	bool     m_enabled;
+	uint32_t m_count;
+};
+
+
 // polyphonic synth implementation
 
 class synthv1_impl
@@ -765,6 +799,11 @@ public:
 
 	void reset();
 
+	void midiInEnabled(bool on);
+	uint32_t midiInCount();
+
+	void directNoteOn(int note, int vel);
+
 	synthv1_wave dco1_wave1, dco1_wave2;
 	synthv1_wave dco2_wave1, dco2_wave2;
 
@@ -820,6 +859,7 @@ private:
 	synthv1_config   m_config;
 	synthv1_controls m_controls;
 	synthv1_programs m_programs;
+	synthv1_midi_in  m_midi_in;
 
 	uint16_t m_nchannels;
 	float    m_srate;
@@ -866,6 +906,10 @@ private:
 
 	synthv1_reverb m_reverb;
 	synthv1_phasor m_phasor;
+
+	volatile int m_direct_chan;
+	volatile int m_direct_note;
+	volatile int m_direct_vel;
 };
 
 
@@ -903,7 +947,7 @@ synthv1_voice::synthv1_voice ( synthv1_impl *pImpl ) :
 
 synthv1_impl::synthv1_impl (
 	synthv1 *pSynth, uint16_t nchannels, float srate )
-	: m_controls(pSynth), m_programs(pSynth), m_bpm(180.0f)
+	: m_controls(pSynth), m_programs(pSynth), m_midi_in(pSynth), m_bpm(180.0f)
 {
 	// max env. stage length (default)
 	m_dco1.envtime0 = m_dco2.envtime0 = 0.0001f * MAX_ENV_MSECS;
@@ -1709,6 +1753,9 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 
 	// process pending controllers...
 	m_controls.process_dequeue();
+
+	// asynchronous event notification...
+	m_midi_in.schedule_event();
 }
 
 
@@ -1735,11 +1782,6 @@ void synthv1_impl::allControllersOff_2 (void)
 
 void synthv1_impl::allNotesOff (void)
 {
-#if 0
-	allNotesOff_1();
-	allNotesOff_2();
-#else
-
 	synthv1_voice *pv = m_play_list.next();
 	while (pv) {
 		if (pv->note1 >= 0)
@@ -1758,7 +1800,7 @@ void synthv1_impl::allNotesOff (void)
 	m_aux1.reset();
 	m_aux2.reset();
 
-#endif
+	m_direct_chan = m_direct_note = m_direct_vel = -1;
 }
 
 void synthv1_impl::allNotesOff_1 (void)
@@ -1861,6 +1903,21 @@ void synthv1_impl::allSoundOff (void)
 }
 
 
+// direct note-on triggered on next cycle...
+void synthv1_impl::directNoteOn ( int note, int vel )
+{
+	if (vel > 0) {
+		const int ch1 = int(*m_def1.channel);
+		const int ch2 = int(*m_def2.channel);
+		m_direct_chan = (ch1 > 0 ? ch1 - 1 : (ch2 > 0 ? ch2 - 1 : 0)) & 0x0f;
+		m_direct_note = note;
+		m_direct_vel  = vel;
+	} else {
+		m_direct_vel  = 0;
+	}
+}
+
+
 // controllers accessor
 
 synthv1_controls *synthv1_impl::controls (void)
@@ -1932,6 +1989,19 @@ void synthv1_impl::reset (void)
 }
 
 
+// MIDI input asynchronous status notification accessors
+
+void synthv1_impl::midiInEnabled ( bool on )
+{
+	m_midi_in.enabled(on);
+}
+
+uint32_t synthv1_impl::midiInCount (void)
+{
+	return m_midi_in.count();
+}
+
+
 // synthesize
 
 void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
@@ -1949,6 +2019,20 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		::memset(outs[k], 0, nframes * sizeof(float));
 	}
 
+	// process direct note on/off...
+	if (m_direct_chan >= 0 && m_direct_note >= 0 && m_direct_vel >= 0) {
+		struct note_data { uint8_t status, note, vel; } data;
+		data.status = (m_direct_vel > 0 ? 0x90 : 0x80) | m_direct_chan;
+		data.note = m_direct_note;
+		data.vel = m_direct_vel;
+		process_midi((uint8_t *) &data, sizeof(data));
+		if (m_direct_vel == 0) {
+			m_direct_chan = -1;
+			m_direct_note = -1;
+		}
+		m_direct_vel = -1;
+	}
+
 	// controls
 	const float lfo1_freq
 		= get_bpm(*m_lfo1.bpm) / (60.01f - *m_lfo1.rate * 60.0f);
@@ -2292,7 +2376,6 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
-
 //-------------------------------------------------------------------------
 // synthv1 - decl.
 //
@@ -2422,4 +2505,26 @@ void synthv1::reset (void)
 }
 
 
+// MIDI input asynchronous status notification accessors
+
+void synthv1::midiInEnabled ( bool on )
+{
+	m_pImpl->midiInEnabled(on);
+}
+
+
+uint32_t synthv1::midiInCount (void)
+{
+	return m_pImpl->midiInCount();
+}
+
+
+// MIDI direct note on/off triggering
+
+void synthv1::directNoteOn ( int note, int vel )
+{
+	m_pImpl->directNoteOn(note, vel);
+}
+
+
 // end of synthv1.cpp
