@@ -1,7 +1,7 @@
 // synthv1.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -216,6 +216,11 @@ public:
 
 	virtual void updatePreset(bool bDirty) = 0;
 
+	void midiInEnabled(bool on);
+	uint32_t midiInCount();
+
+	void directNoteOn(int note, int vel);
+
 private:
 
 	synthv1_impl *m_pImpl;
