@@ -1,7 +1,7 @@
 // synthv1_ui.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -72,4 +72,22 @@ void synthv1_ui::updatePreset ( bool bDirty )
 }
 
 
+void synthv1_ui::midiInEnabled ( bool bEnabled )
+{
+	m_pSynth->midiInEnabled(bEnabled);
+}
+
+
+uint32_t synthv1_ui::midiInCount (void)
+{
+	return m_pSynth->midiInCount();
+}
+
+
+void synthv1_ui::directNoteOn ( int note, int vel )
+{
+	m_pSynth->directNoteOn(note, vel);
+}
+
+
 // end of synthv1_ui.cpp
