@@ -1,7 +1,7 @@
 // synthv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,6 +30,10 @@
 
 #include <QMessageBox>
 #include <QDir>
+#include <QTimer>
+
+#include <QShowEvent>
+#include <QHideEvent>
 
 
 //-------------------------------------------------------------------------
@@ -124,7 +128,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QStringList states;
 	states << tr("Off");
 	states << tr("On");
-
+#if 0
 	m_ui.Dco1Bandl1Knob->insertItems(0, states);
 	m_ui.Dco1Bandl2Knob->insertItems(0, states);
 	m_ui.Dco2Bandl1Knob->insertItems(0, states);
@@ -135,7 +139,10 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
-
+#else
+	m_ui.Lfo1SyncKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+	m_ui.Lfo2SyncKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+#endif
 	// Special values
 	const QString& sOff = states.first();
 	m_ui.Dco1RingModKnob->setSpecialValueText(sOff);
@@ -194,7 +201,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Lfo1BpmKnob->setScale(1.0f);
 	m_ui.Lfo1BpmKnob->setMinimum(0.0f);
 	m_ui.Lfo1BpmKnob->setMaximum(360.0f);
-	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
+//	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
 	m_ui.Lfo1SweepKnob->setMinimum(-1.0f);
 	m_ui.Lfo1SweepKnob->setMaximum(+1.0f);
 	m_ui.Lfo1CutoffKnob->setMinimum(-1.0f);
@@ -213,7 +220,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Lfo2BpmKnob->setScale(1.0f);
 	m_ui.Lfo2BpmKnob->setMinimum(0.0f);
 	m_ui.Lfo2BpmKnob->setMaximum(360.0f);
-	m_ui.Lfo2BpmKnob->setSingleStep(1.0f);
+//	m_ui.Lfo2BpmKnob->setSingleStep(1.0f);
 	m_ui.Lfo2SweepKnob->setMinimum(-1.0f);
 	m_ui.Lfo2SweepKnob->setMaximum(+1.0f);
 	m_ui.Lfo2CutoffKnob->setMinimum(-1.0f);
@@ -238,11 +245,14 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.Def1ChannelKnob->insertItems(0, channels);
 	m_ui.Def2ChannelKnob->insertItems(0, channels);
-
+#if 0
 	// Mono switches
 	m_ui.Def1MonoKnob->insertItems(0, states);
 	m_ui.Def2MonoKnob->insertItems(0, states);
-
+#else
+	m_ui.Def1MonoKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+	m_ui.Def2MonoKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+#endif
 	// Output (stereo-)width limits.
 	m_ui.Out1WidthKnob->setMinimum(-1.0f);
 	m_ui.Out1WidthKnob->setMaximum(+1.0f);
@@ -261,7 +271,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Del1BpmKnob->setScale(1.0f);
 	m_ui.Del1BpmKnob->setMinimum(0.0f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
-	m_ui.Del1BpmKnob->setSingleStep(1.0f);
+//	m_ui.Del1BpmKnob->setSingleStep(1.0f);
 
 	// Reverb (stereo-)width limits.
 	m_ui.Rev1WidthKnob->setMinimum(-1.0f);
@@ -767,13 +777,15 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	if (pConfig) {
 		synthv1widget_dial::setDialMode(
 			synthv1widget_dial::DialMode(pConfig->iKnobDialMode));
+		synthv1widget_edit::setEditMode(
+			synthv1widget_edit::EditMode(pConfig->iKnobEditMode));
 	}
 
 	// Epilog.
 	// QWidget::adjustSize();
 
 	m_ui.StatusBar->showMessage(tr("Ready"), 5000);
-	m_ui.StatusBar->setModified(false);
+	m_ui.StatusBar->modified(false);
 	m_ui.Preset->setDirtyPreset(false);
 }
 
@@ -786,13 +798,11 @@ synthv1widget::~synthv1widget (void)
 }
 
 
-// Create/initialize the scheduler/work notifier.
-void synthv1widget::initSchedNotifier (void)
+// Open/close the scheduler/work notifier.
+void synthv1widget::openSchedNotifier (void)
 {
-	if (m_sched_notifier) {
-		delete m_sched_notifier;
-		m_sched_notifier = NULL;
-	}
+	if (m_sched_notifier)
+		return;
 
 	synthv1_ui *pSynthUi = ui_instance();
 	if (pSynthUi == NULL)
@@ -803,29 +813,61 @@ void synthv1widget::initSchedNotifier (void)
 	QObject::connect(m_sched_notifier,
 		SIGNAL(notify(int, int)),
 		SLOT(updateSchedNotify(int, int)));
+
+	pSynthUi->midiInEnabled(true);
+}
+
+
+void synthv1widget::closeSchedNotifier (void)
+{
+	if (m_sched_notifier) {
+		delete m_sched_notifier;
+		m_sched_notifier = NULL;
+	}
+
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi)
+		pSynthUi->midiInEnabled(false);
+}
+
+
+// Show/hide widget handlers.
+void synthv1widget::showEvent ( QShowEvent *pShowEvent )
+{
+	QWidget::showEvent(pShowEvent);
+
+	openSchedNotifier();
+}
+
+
+void synthv1widget::hideEvent ( QHideEvent *pHideEvent )
+{
+	closeSchedNotifier();
+
+	QWidget::hideEvent(pHideEvent);
 }
 
 
 // Param kbob (widget) map accesors.
-void synthv1widget::setParamKnob ( synthv1::ParamIndex index, synthv1widget_knob *pKnob )
+void synthv1widget::setParamKnob ( synthv1::ParamIndex index, synthv1widget_param *pParam )
 {
-	pKnob->setDefaultValue(synthv1_param::paramDefaultValue(index));
+	pParam->setDefaultValue(synthv1_param::paramDefaultValue(index));
 
-	m_paramKnobs.insert(index, pKnob);
-	m_knobParams.insert(pKnob, index);
+	m_paramKnobs.insert(index, pParam);
+	m_knobParams.insert(pParam, index);
 
-	QObject::connect(pKnob,
+	QObject::connect(pParam,
 		SIGNAL(valueChanged(float)),
 		SLOT(paramChanged(float)));
 
-	pKnob->setContextMenuPolicy(Qt::CustomContextMenu);
+	pParam->setContextMenuPolicy(Qt::CustomContextMenu);
 
-	QObject::connect(pKnob,
+	QObject::connect(pParam,
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(paramContextMenu(const QPoint&)));
 }
 
-synthv1widget_knob *synthv1widget::paramKnob ( synthv1::ParamIndex index ) const
+synthv1widget_param *synthv1widget::paramKnob ( synthv1::ParamIndex index ) const
 {
 	return m_paramKnobs.value(index, NULL);
 }
@@ -837,9 +879,9 @@ void synthv1widget::setParamValue (
 {
 	++m_iUpdate;
 
-	synthv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob)
-		pKnob->setValue(fValue, bDefault);
+	synthv1widget_param *pParam = paramKnob(index);
+	if (pParam)
+		pParam->setValue(fValue, bDefault);
 
 	updateParamEx(index, fValue);
 
@@ -850,9 +892,9 @@ float synthv1widget::paramValue ( synthv1::ParamIndex index ) const
 {
 	float fValue = 0.0f;
 
-	synthv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob) {
-		fValue = pKnob->value();
+	synthv1widget_param *pParam = paramKnob(index);
+	if (pParam) {
+		fValue = pParam->value();
 	} else {
 		synthv1_ui *pSynthUi = ui_instance();
 		if (pSynthUi)
@@ -869,14 +911,14 @@ void synthv1widget::paramChanged ( float fValue )
 	if (m_iUpdate > 0)
 		return;
 
-	synthv1widget_knob *pKnob = qobject_cast<synthv1widget_knob *> (sender());
-	if (pKnob) {
-		const synthv1::ParamIndex index = m_knobParams.value(pKnob);
+	synthv1widget_param *pParam = qobject_cast<synthv1widget_param *> (sender());
+	if (pParam) {
+		const synthv1::ParamIndex index = m_knobParams.value(pParam);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
-			.arg(pKnob->toolTip())
-			.arg(pKnob->valueText()), 5000);
+			.arg(pParam->toolTip())
+			.arg(pParam->valueText()), 5000);
 		updateDirtyPreset(true);
 	}
 }
@@ -923,14 +965,14 @@ void synthv1widget::updateSchedParam ( synthv1::ParamIndex index, float fValue )
 {
 	++m_iUpdate;
 
-	synthv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob) {
-		pKnob->setValue(fValue, false);
+	synthv1widget_param *pParam = paramKnob(index);
+	if (pParam) {
+		pParam->setValue(fValue, false);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
-			.arg(pKnob->toolTip())
-			.arg(pKnob->valueText()), 5000);
+			.arg(pParam->toolTip())
+			.arg(pParam->valueText()), 5000);
 		updateDirtyPreset(true);
 	}
 
@@ -952,9 +994,9 @@ void synthv1widget::resetParams (void)
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		float fValue = synthv1_param::paramDefaultValue(index);
-		synthv1widget_knob *pKnob = paramKnob(index);
-		if (pKnob)
-			fValue = pKnob->defaultValue();
+		synthv1widget_param *pParam = paramKnob(index);
+		if (pParam && pParam->isDefaultValue())
+			fValue = pParam->defaultValue();
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
 		m_params_ab[i] = fValue;
@@ -978,9 +1020,9 @@ void synthv1widget::swapParams ( bool bOn )
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
-		synthv1widget_knob *pKnob = paramKnob(index);
-		if (pKnob) {
-			const float fOldValue = pKnob->value();
+		synthv1widget_param *pParam = paramKnob(index);
+		if (pParam) {
+			const float fOldValue = pParam->value();
 			const float fNewValue = m_params_ab[i];
 			setParamValue(index, fNewValue);
 			updateParam(index, fNewValue);
@@ -1033,6 +1075,7 @@ void synthv1widget::resetParamValues (void)
 		const float fValue = synthv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
+	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
 	}
 }
@@ -1042,9 +1085,9 @@ void synthv1widget::resetParamValues (void)
 void synthv1widget::resetParamKnobs (void)
 {
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-		synthv1widget_knob *pKnob = paramKnob(synthv1::ParamIndex(i));
-		if (pKnob)
-			pKnob->resetDefaultValue();
+		synthv1widget_param *pParam = paramKnob(synthv1::ParamIndex(i));
+		if (pParam)
+			pParam->resetDefaultValue();
 	}
 }
 
@@ -1140,10 +1183,16 @@ void synthv1widget::updateSchedNotify ( int stype, int sid )
 		return;
 
 #ifdef CONFIG_DEBUG_0
-	qDebug("samplv1widget::updateSchedNotify(%d, %d)", stype, sid);
+	qDebug("samplv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
 #endif
 
 	switch (synthv1_sched::Type(stype)) {
+	case synthv1_sched::MidiIn:
+		if (pSynthUi->midiInCount() > 0) {
+			m_ui.StatusBar->midiInLed(true);
+			QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
+		}
+		break;
 	case synthv1_sched::Controller: {
 		synthv1widget_control *pInstance
 			= synthv1widget_control::getInstance();
@@ -1171,6 +1220,13 @@ void synthv1widget::updateSchedNotify ( int stype, int sid )
 }
 
 
+// MIDI In LED timeout.
+void synthv1widget::midiInLedTimeout (void)
+{
+	m_ui.StatusBar->midiInLed(false);
+}
+
+
 // Menu actions.
 void synthv1widget::helpConfigure (void)
 {
@@ -1215,12 +1271,11 @@ void synthv1widget::helpAbout (void)
 	sText += "<b>" SYNTHV1_TITLE "</b> - " + tr(SYNTHV1_SUBTITLE) + "<br />\n";
 	sText += "<br />\n";
 	sText += tr("Version") + ": <b>" CONFIG_BUILD_VERSION "</b><br />\n";
-	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
-	QStringListIterator iter(list);
-	while (iter.hasNext()) {
+//	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
+	if (!list.isEmpty()) {
 		sText += "<small><font color=\"red\">";
-		sText += iter.next();
-		sText += "</font></small><br />";
+		sText += list.join("<br />\n");
+		sText += "</font></small><br />\n";
 	}
 	sText += "<br />\n";
 	sText += tr("Website") + ": <a href=\"" SYNTHV1_WEBSITE "\">" SYNTHV1_WEBSITE "</a><br />\n";
@@ -1251,7 +1306,7 @@ void synthv1widget::updateDirtyPreset ( bool bDirtyPreset )
 	if (pSynthUi)
 		pSynthUi->updatePreset(bDirtyPreset);
 
-	m_ui.StatusBar->setModified(bDirtyPreset);
+	m_ui.StatusBar->modified(bDirtyPreset);
 	m_ui.Preset->setDirtyPreset(bDirtyPreset);
 }
 
@@ -1259,9 +1314,9 @@ void synthv1widget::updateDirtyPreset ( bool bDirtyPreset )
 // Param knob context menu.
 void synthv1widget::paramContextMenu ( const QPoint& pos )
 {
-	synthv1widget_knob *pKnob
-		= qobject_cast<synthv1widget_knob *> (sender());
-	if (pKnob == NULL)
+	synthv1widget_param *pParam
+		= qobject_cast<synthv1widget_param *> (sender());
+	if (pParam == NULL)
 		return;
 
 	synthv1_ui *pSynthUi = ui_instance();
@@ -1281,9 +1336,9 @@ void synthv1widget::paramContextMenu ( const QPoint& pos )
 		QIcon(":/images/synthv1_control.png"),
 		tr("MIDI &Controller..."));
 
-	if (menu.exec(pKnob->mapToGlobal(pos)) == pAction) {
-		const synthv1::ParamIndex index = m_knobParams.value(pKnob);
-		const QString& sTitle = pKnob->toolTip();
+	if (menu.exec(pParam->mapToGlobal(pos)) == pAction) {
+		const synthv1::ParamIndex index = m_knobParams.value(pParam);
+		const QString& sTitle = pParam->toolTip();
 		synthv1widget_control::showInstance(pControls, index, sTitle, this);
 	}
 }
