@@ -1,7 +1,7 @@
 // synthv1widget_config.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -20,7 +20,7 @@
 *****************************************************************************/
 
 #include "synthv1widget_config.h"
-#include "synthv1widget_knob.h"
+#include "synthv1widget_param.h"
 
 #include <QPushButton>
 #include <QMessageBox>
@@ -52,6 +52,7 @@ synthv1widget_config::synthv1widget_config (
 		m_ui.ProgramsPreviewCheckBox->setChecked(pConfig->bProgramsPreview);
 		m_ui.UseNativeDialogsCheckBox->setChecked(pConfig->bUseNativeDialogs);
 		m_ui.KnobDialModeComboBox->setCurrentIndex(pConfig->iKnobDialMode);
+		m_ui.KnobEditModeComboBox->setCurrentIndex(pConfig->iKnobEditMode);
 		int iCustomStyleTheme = 0;
 		if (!pConfig->sCustomStyleTheme.isEmpty())
 			iCustomStyleTheme = m_ui.CustomStyleThemeComboBox->findText(
@@ -129,6 +130,9 @@ synthv1widget_config::synthv1widget_config (
 	QObject::connect(m_ui.KnobDialModeComboBox,
 		SIGNAL(activated(int)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.KnobEditModeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(optionsChanged()));
 	QObject::connect(m_ui.CustomStyleThemeComboBox,
 		SIGNAL(activated(int)),
 		SLOT(optionsChanged()));
@@ -469,6 +473,9 @@ void synthv1widget_config::accept (void)
 		pConfig->iKnobDialMode = m_ui.KnobDialModeComboBox->currentIndex();
 		synthv1widget_dial::setDialMode(
 			synthv1widget_dial::DialMode(pConfig->iKnobDialMode));
+		pConfig->iKnobEditMode = m_ui.KnobEditModeComboBox->currentIndex();
+		synthv1widget_edit::setEditMode(
+			synthv1widget_edit::EditMode(pConfig->iKnobEditMode));
 		const QString sOldCustomStyleTheme = pConfig->sCustomStyleTheme;
 		if (m_ui.CustomStyleThemeComboBox->currentIndex() > 0)
 			pConfig->sCustomStyleTheme = m_ui.CustomStyleThemeComboBox->currentText();
