@@ -1,7 +1,7 @@
 // synthv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -73,7 +73,7 @@ synthv1widget_jack::synthv1widget_jack ( synthv1_jack *pSynth )
 	m_pSynthUi = new synthv1_ui(m_pSynth);
 
 	// May initialize the scheduler/work notifier.
-	initSchedNotifier();
+	openSchedNotifier();
 
 	// Initialize preset stuff...
 	// initPreset();
